@@ -32,7 +32,14 @@ using L<JSON::Unmarshal|https://github.com/tadzik/JSON-Unmarshal>.
 
 It only outputs the "public" attributes (that is those with accessors
 created by declaring them with the '.' twigil. Attributes without acccessors
-are ignored.
+are ignored.  
+
+If you want to ignore any attributes without a value you can use the
+:skip-null adverb to C<marshal>, which will supress the marshalling of
+any undefined attributes.  Additionally if you want a finer-grained control
+over this behaviour there is a 'json-skip-null' attribute trait which
+will cause the specific attribute to be skipped if it isn't defined irrespective
+of the C<skip-null>.
 
 To allow a finer degree of control of how an attribute is marshalled an
 attribute trait C<is marshalled-by> is provided, this can take either 
@@ -48,7 +55,7 @@ above.
 
 use JSON::Name;
 
-module JSON::Marshal:ver<v0.0.5>:auth<github:jonathanstowe> {
+module JSON::Marshal:ver<0.0.6>:auth<github:jonathanstowe> {
 
     use JSON::Tiny;
 
@@ -86,6 +93,13 @@ module JSON::Marshal:ver<v0.0.5>:auth<github:jonathanstowe> {
         $attr does CustomMarshallerMethod;
         $attr.marshaller = $marshalled-by;
     }
+
+    role SkipNull {
+    }
+
+    multi sub trait_mod:<is> (Attribute $attr, :$json-skip-null!) is export {
+        $attr does SkipNull;
+    }
     
     multi sub _marshal(Cool $value) {
         $value;
@@ -110,7 +124,7 @@ module JSON::Marshal:ver<v0.0.5>:auth<github:jonathanstowe> {
         @ret;
     }
     
-    multi sub _marshal(Mu $obj) returns Hash {
+    multi sub _marshal(Mu $obj, Bool :$skip-null) returns Hash {
         my %ret;
         for $obj.^attributes -> $attr {
             if $attr.has_accessor {
@@ -121,11 +135,13 @@ module JSON::Marshal:ver<v0.0.5>:auth<github:jonathanstowe> {
                     $attr.name.substr(2); # lose the sigil
                 }
                 my $value = $attr.get_value($obj);
-                %ret{$name} = do if $attr ~~ CustomMarshaller {
-                    $attr.marshal($value, $obj);
-                }
-                else {
-                    _marshal($value);
+                if serialise-ok($attr, $value, $skip-null) {
+                    %ret{$name} = do if $attr ~~ CustomMarshaller {
+                        $attr.marshal($value, $obj);
+                    }
+                    else {
+                        _marshal($value);
+                    }
                 }
 
             }
@@ -133,8 +149,18 @@ module JSON::Marshal:ver<v0.0.5>:auth<github:jonathanstowe> {
         %ret;
     }
 
-    sub marshal(Any $obj) returns Str is export {
-        my $ret = _marshal($obj);
+    sub serialise-ok(Attribute $attr, $value, Bool $skip-null ) returns Bool {
+        my $rc = True;
+        if $skip-null || ( $attr ~~ SkipNull ) {
+            if not $value.defined {
+                $rc = False;
+            }
+        }
+        $rc;
+    }
+
+    sub marshal(Any $obj, Bool :$skip-null) returns Str is export {
+        my $ret = _marshal($obj, :$skip-null);
         to-json($ret);
     }
 }
